#pragma once

#include "texture_atlas.hpp"
#include "vulkan_allocator.hpp"

#include <gz-util/string/utility.hpp>
#include <string>
#include <unordered_map>

#include <ft2build.h>
#include FT_FREETYPE_H

#include <gz-util/log.hpp>

namespace gz::vlk {
    struct CharInfo {
        // TODO is atlas member needed?
        uint32_t atlas;
        glm::vec2 texCoordTopLeft;
        glm::vec2 texCoordBottomRight;
    };
    /// Handle
    using Font = uint16_t;
    struct FontInfo {
        FontInfo(TextureAtlas&& atlas) : atlas(atlas) {};
        FT_Face face;
        TextureAtlas atlas;
        using CharInfos = std::unordered_map<char, CharInfo>;
        CharInfos charInfos;
    };


    class FontManager {
        public:
            FontManager(VulkanInstance& instance, const std::string&& fontDir);
            /**
             * @brief Load a font
             * @details
             *  -# load face
             *  -# set the charmap to unicode (throw if not found)
             */
            ~FontManager();
            FontManager(const FontManager& other) = delete;
            FontManager(FontManager&& other) = delete;
            Font getFont(const std::string& fontname);
			const std::unordered_map<std::string, Font>& getFonts() const { return fonts; }

            void getTexCoords(Font font, const char c, glm::vec2& texCoords);
			const vk::DescriptorSet& getDescriptorSet() const { return descriptorSet; }
			const vk::DescriptorSetLayout& getDescriptorSetLayout() const { return descriptorSetLayout; }
        private:
            Font loadFont(const std::string& fontname);
            VulkanInstance& vk;
            FT_Library freetype;
            std::string fontDir;
            Log fLog;

            void addChars(Font font, const std::string& chars);
        /**
         * @name Font management
         */
            /// @{
            /// Font name to handle
            std::unordered_map<std::string, Font> fonts;
            /// Font handle as index
            std::vector<FontInfo> fontInfos;
            /// @}
        /**
         * @name Create desciptors
         * @todo Currently the same as in TextureManager -> merge?
         * @see fm_descriptors
         */
            /// @{
            vk::DescriptorSetLayout descriptorSetLayout;
            vk::DescriptorPool descriptorPool;
            vk::DescriptorSet descriptorSet;
            void createDescriptorResources();
            /// @}
    };
}
